@@ -1,14 +1,9 @@
 import threading
 from html.parser import HTMLParser
 
-import django
-
 # A global lock, showing whether linkcheck is busy
 update_lock = threading.Lock()
 
-if django.VERSION <= (3, 2):
-    default_app_config = 'linkcheck.apps.LinkcheckConfig'
-
 
 class Lister(HTMLParser):
 
@@ -34,7 +29,7 @@ class URLLister(Lister):
         elif tag == 'img' and self.in_a:
             src = [v for k, v in attrs if k == 'src']
             if src:
-                self.text += ' [image:%s] ' % src[0]
+                self.text += f' [image:{src[0]}] '
 
     def handle_endtag(self, tag):
         if tag == 'a' and self.in_a:
@@ -52,7 +47,7 @@ class ImageLister(Lister):
 
     def handle_starttag(self, tag, attrs):
         if tag == 'img':
-            src = [v for k, v in attrs if k=='src']
+            src = [v for k, v in attrs if k == 'src']
             if src:
                 self.urls.append(('', src[0]))
 
@@ -67,17 +62,17 @@ class AnchorLister(HTMLParser):
         self.names = []
 
     def handle_starttag(self, tag, attributes):
-        name = [v for k, v in attributes if k=='id']
+        name = [v for k, v in attributes if k == 'id']
         if name:
             self.names.append(name[0])
         if tag == 'a':
-            name = [v for k, v in attributes if k=='name']
+            name = [v for k, v in attributes if k == 'name']
             if name:
                 self.names.append(name[0])
 
 
 def parse(obj, field, parser):
-    html = getattr(obj,field)
+    html = getattr(obj, field)
     if html:
         parser.feed(html)
         parser.close()
@@ -121,6 +116,7 @@ class Linklist:
 
     object_filter = None
     object_exclude = None
+    filter_callable = None
 
     def __get(self, name, obj, default=None):
         try:
@@ -191,6 +187,8 @@ class Linklist:
             objects = objects.filter(**cls.object_filter).distinct()
         if cls.object_exclude:
             objects = objects.exclude(**cls.object_exclude).distinct()
+        if cls.filter_callable:
+            objects = cls.filter_callable(objects)
         return objects
 
     def get_linklist(self, extra_filter=None):
