@@ -12,6 +12,8 @@ class BaseLinkcheckConfig(AppConfig):
     name = 'linkcheck'
     verbose_name = "Linkcheck"
 
+    default_auto_field = "django.db.models.BigAutoField"
+
     all_linklists = {}
 
     def ready(self):
@@ -20,19 +22,19 @@ class BaseLinkcheckConfig(AppConfig):
     def build_linklists(self):
         """Autodiscovery of linkLists"""
         for app in apps.get_app_configs():
-            module_name = "%s.linklists" % app.name
+            module_name = f"{app.name}.linklists"
             if not importlib.util.find_spec(module_name):
                 continue
             the_module = importlib.import_module(module_name)
             try:
                 for k in the_module.linklists.keys():
                     if k in self.all_linklists.keys():
-                        raise AlreadyRegistered('The key %s is already registered in all_linklists' % k)
+                        raise AlreadyRegistered(f'The key {k} is already registered in all_linklists')
 
-                for l in the_module.linklists.values():
-                    for l2 in self.all_linklists.values():
-                        if l.model == l2.model:
-                            raise AlreadyRegistered('The LinkList %s is already registered in all_linklists' % l)
+                for link_list in the_module.linklists.values():
+                    for link_list2 in self.all_linklists.values():
+                        if link_list.model == link_list2.model:
+                            raise AlreadyRegistered(f"The LinkList {link_list} is already registered in all_linklists")
                 self.all_linklists.update(the_module.linklists)
             except AttributeError:
                 pass
