@@ -19,7 +19,7 @@ class Command(BaseCommand):
 
         self.stdout.write("Checking all internal links.")
         if limit != -1:
-            self.stdout.write("Will run maximum of %s checks this run." % limit)
+            self.stdout.write(f"Will run maximum of {limit} checks this run.")
 
         check_count = check_links(limit=limit, check_external=False)
-        return "%s internal URLs have been checked." % (check_count)
+        return f"{check_count} internal URLs have been checked."
