@@ -2,6 +2,7 @@ import logging
 import os.path
 import re
 from datetime import timedelta
+from http import HTTPStatus
 from urllib.parse import unquote
 
 import requests
@@ -14,8 +15,8 @@ from django.test.utils import modify_settings
 from django.utils.encoding import iri_to_uri
 from django.utils.functional import cached_property
 from django.utils.timezone import now
+from django.utils.translation import gettext as _
 from requests.exceptions import ConnectionError, ReadTimeout
-from requests.models import REDIRECT_STATI
 
 try:
     from reversion.revisions import revision_context_manager
@@ -56,6 +57,11 @@ def html_decode(s):
     return s
 
 
+STATUS_CODE_CHOICES = [(s.value, f'{s.value} {s.phrase}') for s in HTTPStatus]
+DEFAULT_USER_AGENT = f'{settings.SITE_DOMAIN} Linkchecker'
+FALLBACK_USER_AGENT = 'Mozilla/5.0 (Windows NT 10.0; rv:91.0) Gecko/20100101 Firefox/91.0'
+
+
 class Url(models.Model):
     """
     Represents a distinct URL found somewhere in the models registered with linkcheck
@@ -64,23 +70,32 @@ class Url(models.Model):
     # See http://www.boutell.com/newfaq/misc/urllength.html
     url = models.CharField(max_length=MAX_URL_LENGTH, unique=True)
     last_checked = models.DateTimeField(blank=True, null=True)
+    anchor_status = models.BooleanField(null=True)
+    ssl_status = models.BooleanField(null=True)
     status = models.BooleanField(null=True)
+    status_code = models.IntegerField(choices=STATUS_CODE_CHOICES, null=True)
+    redirect_status_code = models.IntegerField(choices=STATUS_CODE_CHOICES, null=True)
     message = models.CharField(max_length=1024, blank=True, null=True)
+    error_message = models.CharField(max_length=1024, default='', blank=True)
     redirect_to = models.TextField(blank=True)
 
+    @property
+    def redirect_ok(self):
+        return self.redirect_status_code < 300 if self.redirect_status_code else None
+
     @property
     def type(self):
         if self.external:
             return 'external'
-        if self.url.startswith('mailto'):
+        if self.url.startswith('mailto:'):
             return 'mailto'
-        if self.url.startswith('tel'):
+        if self.url.startswith('tel:'):
             return 'phone'
-        elif self.url == '':
+        elif self.internal_url == '':
             return 'empty'
-        elif self.url.startswith('#'):
+        elif self.internal_url.startswith('#'):
             return 'anchor'
-        elif self.url.startswith(MEDIA_PREFIX):
+        elif self.internal_url.startswith(MEDIA_PREFIX):
             return 'file'
         elif self.internal_url.startswith('/'):
             return 'internal'
@@ -95,12 +110,57 @@ class Url(models.Model):
     def anchor(self):
         return self.url.split('#')[1] if self.has_anchor else None
 
+    @property
+    def anchor_message(self):
+        if not self.has_anchor or not self.last_checked:
+            return ''
+        if self.anchor == '':
+            return _('Working empty anchor')
+        if self.anchor_status is None:
+            return _('Anchor could not be checked')
+        elif self.anchor_status is False:
+            return _('Broken anchor')
+        return _('Working anchor')
+
+    @property
+    def ssl_message(self):
+        if self.internal:
+            return ''
+        if self.external_url.startswith('http://'):
+            return _('Insecure link')
+        if self.ssl_status is None:
+            return _('SSL certificate could not be checked')
+        elif self.ssl_status is False:
+            return _('Broken SSL certificate')
+        return _('Valid SSL certificate')
+
     @property
     def get_message(self):
-        if self.last_checked:
-            return self.message
-        else:
-            return "URL Not Yet Checked"
+        if not self.last_checked:
+            return _('URL Not Yet Checked')
+        elif self.type == 'empty':
+            return _('Empty link')
+        elif self.type == 'invalid':
+            return _('Invalid URL')
+        elif self.type == 'mailto':
+            return '{} ({})'.format(_("Email link"), _("not automatically checked"))
+        elif self.type == 'phone':
+            return '{} ({})'.format(_("Phone number link"), _("not automatically checked"))
+        elif self.type == 'anchor':
+            return '{} ({})'.format(_("Anchor link"), _("not automatically checked"))
+        elif self.type == 'file':
+            return _('Working file link') if self.status else _('Missing file')
+        elif not self.status_code:
+            return self.error_message
+        elif self.status_code < 300:
+            return _('Working external link') if self.external else _('Working internal link')
+        elif self.status_code < 400:
+            permanent = self.status_code in [HTTPStatus.MOVED_PERMANENTLY, HTTPStatus.PERMANENT_REDIRECT]
+            if self.redirect_ok:
+                return _('Working permanent redirect') if permanent else _('Working temporary redirect')
+            else:
+                return _('Broken permanent redirect') if permanent else _('Broken temporary redirect')
+        return _('Broken external link') if self.external else _('Broken internal link')
 
     @property
     def colour(self):
@@ -194,6 +254,20 @@ class Url(models.Model):
         """
         return not self.internal
 
+    def reset_for_check(self):
+        """
+        Reset all fields which depend on the status after checking a URL.
+        This is done to ensure that results from the last check do not remain if the fields are not overwritten.
+        """
+        # Reset all database fields
+        self.anchor_status = None
+        self.status = None
+        self.status_code = None
+        self.redirect_status_code = None
+        self.ssl_status = None
+        self.error_message = ''
+        self.message = ''
+
     def check_url(self, check_internal=True, check_external=True, external_recheck_interval=EXTERNAL_RECHECK_INTERVAL):
         """
         Return:
@@ -202,19 +276,13 @@ class Url(models.Model):
          * None if the link was not checked
         """
 
-        self.status = False
-
         if check_internal and self.internal:
-            self.check_internal()
-
+            return self.check_internal()
         elif check_external and self.external:
-            self.check_external(external_recheck_interval)
-
+            return self.check_external(external_recheck_interval)
         else:
             return None
 
-        return self.status
-
     def check_internal(self):
         """
         Check an internal URL
@@ -225,21 +293,22 @@ class Url(models.Model):
 
         logger.debug('checking internal link: %s', self.internal_url)
 
+        # Reset all fields in case they were already set
+        self.reset_for_check()
+
         from linkcheck.utils import LinkCheckHandler
 
         if self.type == 'empty':
+            self.status = False
             self.message = 'Empty link'
 
         elif self.type == 'mailto':
-            self.status = None
             self.message = 'Email link (not automatically checked)'
 
         elif self.type == 'phone':
-            self.status = None
             self.message = 'Phone number (not automatically checked)'
 
         elif self.type == 'anchor':
-            self.status = None
             self.message = 'Link to within the same page (not automatically checked)'
 
         elif self.type == 'file':
@@ -247,25 +316,8 @@ class Url(models.Model):
             # This will break quite easily for alternate setups
             path = settings.MEDIA_ROOT + unquote(self.internal_url)[len(MEDIA_PREFIX) - 1:]
             decoded_path = html_decode(path)
-            if os.path.exists(path) or os.path.exists(decoded_path):
-                self.message = 'Working file link'
-                self.status = True
-            else:
-                self.message = 'Missing Document'
-
-        elif getattr(self, '_internal_hash', False) and getattr(self, '_instance', None):
-            # This is a hash link pointing to itself
-            hash = self._internal_hash
-            instance = self._instance
-            if hash == '#':  # special case, point to #
-                self.message = 'Working internal hash anchor'
-                self.status = True
-            else:
-                hash = hash[1:]  # '#something' => 'something'
-                html_content = ''
-                for field in instance._linklist.html_fields:
-                    html_content += getattr(instance, field, '')
-                self._check_anchor(hash, html_content)
+            self.status = os.path.exists(path) or os.path.exists(decoded_path)
+            self.message = 'Working file link' if self.status else 'Missing Document'
 
         elif self.type == 'internal':
             old_prepend_setting = settings.PREPEND_WWW
@@ -274,19 +326,20 @@ class Url(models.Model):
             c.handler = LinkCheckHandler()
             with modify_settings(ALLOWED_HOSTS={'append': 'testserver'}):
                 response = c.get(self.internal_url)
-            if response.status_code == 200:
+            self.status_code = response.status_code
+            if response.status_code < 300:
                 self.message = 'Working internal link'
                 self.status = True
-            elif response.status_code == 302 or response.status_code == 301:
+            elif response.status_code < 400:
                 redirect_type = "permanent" if response.status_code == 301 else "temporary"
-                with modify_settings(ALLOWED_HOSTS={'append': 'testserver'}):
-                    response = c.get(self.internal_url, follow=True)
-                if response.status_code == 200:
-                    self.message = f'Working {redirect_type} redirect'
-                    self.status = True
-                else:
-                    self.message = f'Broken {redirect_type} redirect'
+                response = c.get(self.internal_url, follow=True)
+                self.redirect_to, _ = response.redirect_chain[-1]
+                self.redirect_status_code = response.status_code
+                self.status = response.status_code < 300
+                redirect_result = "Working" if self.status else "Broken"
+                self.message = f'{redirect_result} {redirect_type} redirect'
             else:
+                self.status = False
                 self.message = 'Broken internal link'
 
             # Check the anchor (if it exists)
@@ -294,6 +347,7 @@ class Url(models.Model):
 
             settings.PREPEND_WWW = old_prepend_setting
         else:
+            self.status = False
             self.message = 'Invalid URL'
 
         if USE_REVERSION:
@@ -302,6 +356,7 @@ class Url(models.Model):
 
         self.last_checked = now()
         self.save()
+        return self.status
 
     def check_external(self, external_recheck_interval=EXTERNAL_RECHECK_INTERVAL):
         """
@@ -321,62 +376,98 @@ class Url(models.Model):
             )
             return self.status
 
+        # Reset all fields in case they were already set
+        self.reset_for_check()
+
         request_params = {
             'allow_redirects': True,
-            'headers': {'User-Agent': f"http://{settings.SITE_DOMAIN} Linkchecker"},
+            'headers': {'User-Agent': DEFAULT_USER_AGENT},
             'timeout': LINKCHECK_CONNECTION_ATTEMPT_TIMEOUT,
             'verify': True,
         }
         try:
             try:
+                # If no exceptions occur, the SSL certificate is valid
+                if self.external_url.startswith('https://'):
+                    self.ssl_status = True
                 # At first try a HEAD request
-                response = requests.head(self.external_url, **request_params)
+                fetch = requests.head
+                response = fetch(self.external_url, **request_params)
             except ConnectionError as e:
                 # This error could also be caused by an incomplete root certificate bundle,
                 # so let's retry without verifying the certificate
                 if "unable to get local issuer certificate" in str(e):
                     request_params['verify'] = False
-                    response = requests.head(self.external_url, **request_params)
+                    self.ssl_status = None
+                    response = fetch(self.external_url, **request_params)
                 else:
                     # Re-raise exception if it's definitely not a false positive
                     raise
             # If HEAD is not allowed, let's try with GET
-            if response.status_code >= 400:
+            if response.status_code in [HTTPStatus.BAD_REQUEST, HTTPStatus.METHOD_NOT_ALLOWED]:
                 logger.debug('HEAD is not allowed, retry with GET')
-                response = requests.get(self.external_url, **request_params)
+                fetch = requests.get
+                response = fetch(self.external_url, **request_params)
+            # If access is denied, possibly the user agent is blocked
+            if response.status_code == HTTPStatus.FORBIDDEN:
+                logger.debug('Forbidden, retry with different user agent')
+                request_params['headers'] = {'User-Agent': FALLBACK_USER_AGENT}
+                response = fetch(self.external_url, **request_params)
             # If URL contains hash anchor and is a valid HTML document, let's repeat with GET
             elif (
                 self.has_anchor and
                 response.ok and
+                fetch == requests.head and
                 'text/html' in response.headers.get('content-type')
             ):
                 logger.debug('Retrieve content for anchor check')
-                response = requests.get(self.external_url, **request_params)
+                fetch = requests.get
+                response = fetch(self.external_url, **request_params)
         except ReadTimeout:
+            self.status = False
             self.message = 'Other Error: The read operation timed out'
+            self.error_message = 'The read operation timed out'
         except ConnectionError as e:
-            self.message = format_connection_error(e)
+            self.status = False
+            self.message = self.error_message = format_connection_error(e)
+            if 'SSLError' in str(e):
+                self.ssl_status = False
         except Exception as e:
+            self.status = False
             self.message = f'Other Error: {e}'
+            self.error_message = str(e)
         else:
+            self.status = response.status_code < 300
             self.message = f"{response.status_code} {response.reason}"
             logger.debug('Response message: %s', self.message)
 
-            if response.ok and response.status_code not in REDIRECT_STATI:
-                self.status = True
-                # If initial response was a redirect, return the initial return code
-                if response.history:
-                    logger.debug('Redirect history: %r', response.history)
-                    self.message = f"{response.history[0].status_code} {response.history[0].reason}"
-                    self.redirect_to = response.url
+            # If initial response was a redirect, return the initial return code
+            if response.history:
+                logger.debug('Redirect history: %r', response.history)
+                if response.ok:
+                    self.message = f'{response.history[0].status_code} {response.history[0].reason}'
+                self.redirect_to = response.url
+                self.redirect_status_code = response.status_code
+                self.status_code = response.history[0].status_code
+            else:
+                self.status_code = response.status_code
+
             # Check the anchor (if it exists)
-            if response.request.method == 'GET':
+            if fetch == requests.get:
                 self.check_anchor(response.text)
             if not request_params['verify']:
                 self.message += ', SSL certificate could not be verified'
 
-        self.last_checked = now()
+        # When a rate limit was hit or the server returned an internal error, do not update
+        # the last_checked date so the result is not cached for EXTERNAL_RECHECK_INTERVAL minutes
+        if (
+            not self.status_code or
+            self.status_code != HTTPStatus.TOO_MANY_REQUESTS and
+            self.status_code < 500
+        ):
+            self.last_checked = now()
         self.save()
+        return self.status
 
     def check_anchor(self, html):
         from linkcheck import parse_anchors
@@ -387,6 +478,7 @@ class Url(models.Model):
         if self.has_anchor:
             # Empty fragment '#' is always valid
             if not self.anchor:
+                self.anchor_status = True
                 self.message += f', working {scope} hash anchor'
             else:
                 try:
@@ -403,11 +495,14 @@ class Url(models.Model):
                         self.status = False
                 else:
                     if self.anchor in names:
+                        self.anchor_status = True
                         self.message += f', working {scope} hash anchor'
                     else:
+                        self.anchor_status = False
                         self.message += f', broken {scope} hash anchor'
                         if not TOLERATE_BROKEN_ANCHOR:
                             self.status = False
+        return self.anchor_status, self.anchor_message
 
 
 class Link(models.Model):
