@@ -1,5 +1,6 @@
 import time
 
+from django.core.exceptions import PermissionDenied
 from django.http import (
     HttpResponse,
     HttpResponsePermanentRedirect,
@@ -16,6 +17,14 @@ def http_response_get_only(request, code):
     return HttpResponse("", status=status)
 
 
+def http_block_user_agent(request, block_head=False):
+    if block_head and request.method == 'HEAD':
+        return HttpResponse('', status=405)
+    if 'Linkchecker' in request.headers.get('User-Agent', ''):
+        raise PermissionDenied()
+    return HttpResponse('')
+
+
 def http_redirect(request, code):
     return HttpResponseRedirect("/http/200/", status=int(code))
 
