@@ -1,12 +1,20 @@
+import logging
+from datetime import timedelta
+
 from django.apps import apps
 from django.db import models
 from django.test.client import ClientHandler
 from django.utils import timezone
 
-from datetime import timedelta
-
+from .linkcheck_settings import (
+    HTML_FIELD_CLASSES,
+    IMAGE_FIELD_CLASSES,
+    MAX_URL_LENGTH,
+    URL_FIELD_CLASSES,
+)
 from .models import Link, Url
-from .linkcheck_settings import MAX_URL_LENGTH, HTML_FIELD_CLASSES, IMAGE_FIELD_CLASSES, URL_FIELD_CLASSES
+
+logger = logging.getLogger(__name__)
 
 
 class LinkCheckHandler(ClientHandler):
@@ -14,13 +22,14 @@ class LinkCheckHandler(ClientHandler):
     # Customize the ClientHandler to allow us removing some middlewares
 
     def load_middleware(self):
-        self.ignore_keywords = ['reversion.middleware','MaintenanceModeMiddleware']
+        self.ignore_keywords = ['reversion.middleware', 'MaintenanceModeMiddleware']
         super().load_middleware()
         new_request_middleware = []
-        
-        #############################_request_middleware#################################
-        # _request_middleware is removed in newer django.
-        if getattr(self, '_request_middleware', None):
+
+        ####################################################
+        # _request_middleware (is removed in newer django) #
+        ####################################################
+        if getattr(self, "_request_middleware", None):
             for method in self._request_middleware:
                 ignored = False
                 for keyword in self.ignore_keywords:
@@ -30,8 +39,10 @@ class LinkCheckHandler(ClientHandler):
                 if not ignored:
                     new_request_middleware.append(method)
             self._request_middleware = new_request_middleware
-        
-        #############################_view_middleware#################################
+
+        ####################
+        # _view_middleware #
+        ####################
         new_view_middleware = []
         for method in self._view_middleware:
             ignored = False
@@ -43,8 +54,10 @@ class LinkCheckHandler(ClientHandler):
                 new_view_middleware.append(method)
         self._view_middleware = new_view_middleware
 
-        #############################_response_middleware#################################
-        if getattr(self, '_response_middleware', None):
+        ##########################
+        # _response_middleware## #
+        ##########################
+        if getattr(self, "_response_middleware", None):
             new_response_middleware = []
             for method in self._response_middleware:
                 ignored = False
@@ -56,9 +69,10 @@ class LinkCheckHandler(ClientHandler):
                     new_response_middleware.append(method)
             self._response_middleware = new_response_middleware
 
-
-        #############################_template_response_middleware#################################
-        if getattr(self, '_template_response_middleware', None):
+        #################################
+        # _template_response_middleware #
+        #################################
+        if getattr(self, "_template_response_middleware", None):
             new_template_response_middleware = []
             for method in self._template_response_middleware:
                 ignored = False
@@ -70,7 +84,9 @@ class LinkCheckHandler(ClientHandler):
                     new_template_response_middleware.append(method)
             self._template_response_middleware = new_template_response_middleware
 
-        #############################_exception_middleware#################################
+        #########################
+        # _exception_middleware #
+        #########################
         new_exception_middleware = []
         for method in self._exception_middleware:
             ignored = False
@@ -88,7 +104,7 @@ def check_links(external_recheck_interval=10080, limit=-1, check_internal=True,
     Return the number of links effectively checked.
     """
 
-    urls = Url.objects.filter(still_exists=True)
+    urls = Url.objects.all()
 
     # An optimization for when check_internal is False
     if not check_internal:
@@ -99,7 +115,7 @@ def check_links(external_recheck_interval=10080, limit=-1, check_internal=True,
     for u in urls:
         status = u.check_url(check_internal=check_internal, check_external=check_external)
         check_count += 1 if status is not None else 0
-        if limit > -1 and check_count >= limit:
+        if -1 < limit <= check_count:
             break
 
     return check_count
@@ -109,7 +125,9 @@ def update_urls(urls, content_type, object_id):
 
     # Structure of urls param is [(field, link text, url), ... ]
 
-    new_urls = new_links = 0
+    urls_created = links_created = 0
+    new_url_ids = set()
+    new_link_ids = set()
 
     for field, link_text, url in urls:
 
@@ -119,6 +137,7 @@ def update_urls(urls, content_type, object_id):
 
         if len(url) > MAX_URL_LENGTH:
             # We cannot handle url longer than MAX_URL_LENGTH at the moment
+            logger.warning('URL exceeding max length will be skipped: %s', url)
             continue
 
         url, url_created = Url.objects.get_or_create(url=url)
@@ -131,12 +150,24 @@ def update_urls(urls, content_type, object_id):
             object_id=object_id,
         )
 
-        url.still_exists = True
-        url.save()
-        new_urls += url_created
-        new_links += link_created
+        # Keep track of how many objects were created
+        urls_created += url_created
+        links_created += link_created
 
-    return new_urls, new_links
+        # Keep track of object ids (no matter if created or existing)
+        new_url_ids.add(url.id)
+        new_link_ids.add(link.id)
+
+    return {
+        "urls": {
+            "created": urls_created,
+            "ids": new_url_ids,
+        },
+        "links": {
+            "created": links_created,
+            "ids": new_link_ids,
+        },
+    }
 
 
 def find_all_links(linklists=None):
@@ -144,10 +175,12 @@ def find_all_links(linklists=None):
     if linklists is None:
         linklists = apps.get_app_config('linkcheck').all_linklists
 
-    all_links_dict = {}
     urls_created = links_created = 0
+    new_url_ids = set()
+    new_link_ids = set()
 
-    Url.objects.all().update(still_exists=False)
+    urls_before = Url.objects.count()
+    links_before = Link.objects.count()
 
     for linklist_name, linklist_cls in linklists.items():
 
@@ -158,19 +191,33 @@ def find_all_links(linklists=None):
             object_id = linklist['object'].id
             urls = linklist['urls'] + linklist['images']
             if urls:
-                new_urls, new_links = update_urls(urls, content_type, object_id)
-                urls_created += new_urls
-                links_created += new_links
-        all_links_dict[linklist_name] = linklists
+                new = update_urls(urls, content_type, object_id)
 
-    deleted = Url.objects.filter(still_exists=False).count()
+                urls_created += new["urls"]["created"]
+                links_created += new["links"]["created"]
 
-    Url.objects.filter(still_exists=False).delete()
+                new_url_ids.update(new["urls"]["ids"])
+                new_link_ids.update(new["links"]["ids"])
+
+    # Delete all urls and links which are no longer part of the link lists
+    Url.objects.all().exclude(id__in=new_url_ids).delete()
+    Link.objects.all().exclude(id__in=new_link_ids).delete()
+
+    # Calculate diff
+    urls_after = Url.objects.count()
+    links_after = Link.objects.count()
 
     return {
-        'urls_deleted': deleted,
-        'urls_created': urls_created,
-        'links_created': links_created,
+        "urls": {
+            "created": urls_created,
+            "deleted": urls_before + urls_created - urls_after,
+            "unchanged": urls_after - urls_created,
+        },
+        "links": {
+            "created": links_created,
+            "deleted": links_before + links_created - links_after,
+            "unchanged": links_after - links_created,
+        },
     }
 
 
@@ -275,7 +322,7 @@ def get_coverage_data():
                         break
             if should_append:
                 all_model_list.append({
-                    'name': '%s.%s' % (model._meta.app_label, model._meta.object_name),
+                    'name': f'{model._meta.app_label}.{model._meta.object_name}',
                     'is_covered': is_model_covered(model),
                     'suggested_config': get_suggested_linklist_config(model),
                 })
